// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "EndlessVendetta/GadgetSystem/GadgetBase.h"
#include "EndlessVendetta/InteractionInterface.h"
#include "GameFramework/Actor.h"
#include "GadgetTutorialStation.generated.h"

USTRUCT(BlueprintType)
struct FGadgetInfo
{
	GENERATED_BODY()

	UPROPERTY(BlueprintReadWrite, Category = "Gadget")
	UTexture2D* GadgetIcon;

	UPROPERTY(BlueprintReadWrite, Category = "Gadget")
	FString GadgetDesc;

	UPROPERTY(BlueprintReadWrite, Category = "Gadget")
	FString GadgetName;
};

UCLASS()
class ENDLESSVENDETTA_API AGadgetTutorialStation : public AActor, public IInteractionInterface
{
	GENERATED_BODY()

	UPROPERTY(EditDefaultsOnly, Category = "Gadgets")
	TArray<TSubclassOf<AGadgetBase>> GadgetsArray;

	UPROPERTY(EditDefaultsOnly, Category = "Gadgets")
	TSubclassOf<UUserWidget> PickGadgetWidgetClass;

	int GadgetIndex = 0;

	UUserWidget* PickGadgetWidget;

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	void Interact() override;\

	void InteractPrompt() override;

	UFUNCTION(BlueprintCallable)
	FGadgetInfo NextGadget();

	UFUNCTION(BlueprintCallable)
	FGadgetInfo PreviousGadget();

	UFUNCTION(BlueprintCallable)
	void SelectGadget();

	UFUNCTION(BlueprintCallable)
	void CloseWidget();

public:
	// Sets default values for this actor's properties
	AGadgetTutorialStation();
	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

};
