﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "InventoryStructs.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "InventoryComponent.generated.h"


UCLASS(ClassGroup=(Custom), meta=(BlueprintSpawnableComponent))
class ENDLESSVENDETTA_API UInventoryComponent : public UActorComponent
{
	GENERATED_BODY()

public:
	// Sets default values for this component's properties
	UInventoryComponent();
	virtual void InitializeComponent() override;

	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Inventory")
	int Columns = 10;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Inventory")
	int Rows = 10;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category="Inventory")
	float MaxPickupDistance = 200.0f;

	
protected:
	// Called when the game starts
	virtual void BeginPlay() override;

public:
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

	DECLARE_DYNAMIC_MULTICAST_DELEGATE(FOnInventoryUpdated);
	UPROPERTY(BlueprintAssignable, Category="Inventory")
	FOnInventoryUpdated OnInventoryUpdated;
	
	UFUNCTION(BlueprintCallable, Category="Inventory")
	bool AddItem(class UBaseItem* Item);
	UFUNCTION(BlueprintCallable, Category="Inventory")
	void ProjectTraceForItem();
	UFUNCTION(BlueprintCallable, Category="Inventory")
	void Pickup();
	UFUNCTION(BlueprintCallable, Category="Inventory")
	bool IsRoomAvailable(class UBaseItem* Item, const int TopLeftIndex);
	UFUNCTION(BlueprintCallable, Category="Inventory")
	FInventoryTile IndexToTile(const int Index) const;
	//UFUNCTION(BlueprintCallable, Category="Inventory")
	TTuple<UBaseItem*, bool> GetItemAtIndex(const int Index);
	UFUNCTION(BlueprintCallable, Category="Inventory")
	int TileToIndex(const FInventoryTile InventoryTile) const;
	UFUNCTION(BlueprintCallable, Category="Inventory")
	void AddItemAt(class UBaseItem* Item, const int TopLeftIndex);
	UFUNCTION(BlueprintCallable, Category="Inventory")
	TMap<UBaseItem*, FInventoryTile> GetAllItems();
	UFUNCTION(BlueprintCallable, Category="Inventory")
	void RemoveItem(class UBaseItem* Item);
	UFUNCTION(BlueprintCallable, Category="Inventory")
	void SpawnItem(class UBaseItem* Item, FVector Location, FRotator Rotation);

private:
	bool IsTileValid(const FInventoryTile InventoryTile) const;
	
	UPROPERTY()
	AEndlessVendettaCharacter* PlayerCharacter;
	UPROPERTY()
	AActor* ItemToPickup;
	bool IsDirty = false;
	UPROPERTY()
	TArray<UBaseItem*> InventoryItems;
};
